@@ -4,7 +4,7 @@
 # convertdlg.py, provides the main dialog and GUI interface
 #
 # ConvertAll, a units conversion program
-# Copyright (C) 2006, Douglas W. Bell
+# Copyright (C) 2010, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
@@ -17,13 +17,17 @@ import os.path
 from PyQt4 import QtCore, QtGui
 try:
     from __main__ import __version__, __author__, helpFilePath, iconPath
+    from __main__ import lang, localEncoding
 except ImportError:
     __version__ = __author__ = '??'
     helpFilePath = None
     iconPath = None
+    lang = ''
+    localEncoding = 'utf-8'
 import unitdata
 from unitgroup import UnitGroup
 from option import Option
+import recentunits
 import unitedit
 import unitlistview
 import numedit
@@ -56,18 +60,23 @@ class ConvertDlg(QtGui.QWidget):
         self.findDlg = None
         self.option = Option('convertall', 20)
         self.option.loadAll(optiondefaults.defaultList)
+        self.recentUnits = recentunits.RecentUnits(self.option)
         try:
             num = ConvertDlg.unitData.readData()
-            print '%d units loaded' % num
         except unitdata.UnitDataError, text:
             QtGui.QMessageBox.warning(self, 'ConvertAll',
-                                      'Error in unit data - %s' % text)
+                                      _('Error in unit data - %s') % text)
             sys.exit(1)
+        try:
+            print (_('%d units loaded') % num).encode(localEncoding)
+        except UnicodeError:
+            print (_('%d units loaded') % num).encode('utf-8')
         self.fromGroup = UnitGroup(ConvertDlg.unitData, self.option)
         self.toGroup = UnitGroup(ConvertDlg.unitData, self.option)
         self.origPal = QtGui.QApplication.palette()
         self.updateColors()
         self.textButtons = []
+        self.recentButtons = []
 
         topLayout = QtGui.QHBoxLayout(self)    # divide main, buttons
         mainLayout = QtGui.QVBoxLayout()
@@ -75,7 +84,7 @@ class ConvertDlg(QtGui.QWidget):
         unitLayout = QtGui.QHBoxLayout()       # unit selection
         mainLayout.addLayout(unitLayout)
 
-        fromBox = QtGui.QGroupBox('From Unit')
+        fromBox = QtGui.QGroupBox(_('From Unit'))
         unitLayout.addWidget(fromBox)
         fromLayout = QtGui.QVBoxLayout(fromBox)
         fromLayout.setSpacing(3)
@@ -91,7 +100,7 @@ class ConvertDlg(QtGui.QWidget):
         self.fromUnitListView.setFocusProxy(self.fromUnitEdit)
         self.addButtons(self.fromGroup, self.fromUnitListView, fromLayout)
 
-        toBox = QtGui.QGroupBox('To Unit')
+        toBox = QtGui.QGroupBox(_('To Unit'))
         unitLayout.addWidget(toBox)
         toLayout = QtGui.QVBoxLayout(toBox)
         toLayout.setSpacing(3)
@@ -109,15 +118,16 @@ class ConvertDlg(QtGui.QWidget):
 
         numberLayout = QtGui.QHBoxLayout()
         mainLayout.addLayout(numberLayout)
-        statusLabel = QtGui.QLabel('Set units')
+        statusLabel = QtGui.QLabel(_('Set units'))
         statusLabel.setFrameStyle(QtGui.QFrame.Panel | QtGui.QFrame.Sunken)
         mainLayout.addWidget(statusLabel)
 
-        fromNumBox = QtGui.QGroupBox('No Unit Set')
+        fromNumBox = QtGui.QGroupBox(_('No Unit Set'))
         numberLayout.addWidget(fromNumBox)
         fromNumLayout = QtGui.QVBoxLayout(fromNumBox)
         self.fromNumEdit = numedit.NumEdit(self.fromGroup, self.toGroup,
-                                           fromNumBox, statusLabel, True)
+                                           fromNumBox, statusLabel,
+                                           self.recentUnits, True)
         fromNumLayout.addWidget(self.fromNumEdit)
         self.connect(self.fromUnitEdit, QtCore.SIGNAL('unitChanged'),
                      self.fromNumEdit.unitUpdate)
@@ -126,19 +136,24 @@ class ConvertDlg(QtGui.QWidget):
         equalsLabel.setFont(QtGui.QFont(self.font().family(), 30))
         numberLayout.addWidget(equalsLabel)
 
-        toNumBox = QtGui.QGroupBox('No Unit Set')
+        toNumBox = QtGui.QGroupBox(_('No Unit Set'))
         numberLayout.addWidget(toNumBox)
         toNumLayout = QtGui.QVBoxLayout(toNumBox)
         self.toNumEdit = numedit.NumEdit(self.toGroup, self.fromGroup,
-                                         toNumBox, statusLabel, False)
+                                         toNumBox, statusLabel,
+                                         self.recentUnits, False)
         toNumLayout.addWidget(self.toNumEdit)
         self.connect(self.toUnitEdit, QtCore.SIGNAL('unitChanged'),
                      self.toNumEdit.unitUpdate)
         self.toNumEdit.setEnabled(False)
-        self.connect(self.fromNumEdit, QtCore.SIGNAL('convertNum(QString &)'),
+        self.connect(self.fromNumEdit, QtCore.SIGNAL('convertNum'),
                      self.toNumEdit.setNum)
-        self.connect(self.toNumEdit, QtCore.SIGNAL('convertNum(QString &)'),
+        self.connect(self.toNumEdit, QtCore.SIGNAL('convertNum'),
                      self.fromNumEdit.setNum)
+        self.connect(self.fromNumEdit, QtCore.SIGNAL('convertNum'),
+                     self.setRecentAvail)
+        self.connect(self.toNumEdit, QtCore.SIGNAL('convertNum'),
+                     self.setRecentAvail)
         self.connect(self.fromNumEdit, QtCore.SIGNAL('convertRqd'),
                      self.toNumEdit.convert)
         self.connect(self.toNumEdit, QtCore.SIGNAL('convertRqd'),
@@ -146,24 +161,24 @@ class ConvertDlg(QtGui.QWidget):
 
         buttonLayout = QtGui.QVBoxLayout()     # major buttons
         topLayout.addLayout(buttonLayout)
-        closeButton = QtGui.QPushButton('&Close')
+        closeButton = QtGui.QPushButton(_('&Close'))
         buttonLayout.addWidget(closeButton)
         closeButton.setFocusPolicy(QtCore.Qt.NoFocus)
         self.connect(closeButton, QtCore.SIGNAL('clicked()'), self.close)
-        finderButton = QtGui.QPushButton('&Unit Finder...')
+        finderButton = QtGui.QPushButton(_('&Unit Finder...'))
         buttonLayout.addWidget(finderButton)
         finderButton.setFocusPolicy(QtCore.Qt.NoFocus)
         self.connect(finderButton, QtCore.SIGNAL('clicked()'), self.showFinder)
-        optionsButton = QtGui.QPushButton('&Options...')
+        optionsButton = QtGui.QPushButton(_('&Options...'))
         buttonLayout.addWidget(optionsButton)
         optionsButton.setFocusPolicy(QtCore.Qt.NoFocus)
         self.connect(optionsButton, QtCore.SIGNAL('clicked()'),
                      self.changeOptions)
-        helpButton = QtGui.QPushButton('&Help...')
+        helpButton = QtGui.QPushButton(_('&Help...'))
         buttonLayout.addWidget(helpButton)
         helpButton.setFocusPolicy(QtCore.Qt.NoFocus)
         self.connect(helpButton, QtCore.SIGNAL('clicked()'), self.help)
-        aboutButton = QtGui.QPushButton('&About...')
+        aboutButton = QtGui.QPushButton(_('&About...'))
         buttonLayout.addWidget(aboutButton)
         aboutButton.setFocusPolicy(QtCore.Qt.NoFocus)
         self.connect(aboutButton, QtCore.SIGNAL('clicked()'), self.about)
@@ -188,13 +203,49 @@ class ConvertDlg(QtGui.QWidget):
         for button in buttons:
             buttonLayout.addWidget(button)
         listView.buttonList = buttons[:]
-        buttons.append(ModButton(unitGroup.clearUnit, None, 'Clear Unit'))
-        upperLayout.addWidget(buttons[-1])
+        buttons.append(ModButton(unitGroup.clearUnit, None, _('Clear Unit')))
+        extraLayout = QtGui.QHBoxLayout()
+        upperLayout.addLayout(extraLayout)
+        extraLayout.addWidget(buttons[-1])
         for but in buttons:
             self.connect(but, QtCore.SIGNAL('stateChg'), listView.relayChange)
             but.setEnabled(False)
             self.textButtons.append(but)
         buttons[-1].setEnabled(True)
+        recentButton = QtGui.QPushButton(_('Recent Unit'))
+        recentButton.setFocusPolicy(QtCore.Qt.NoFocus)
+        recentButton.unitGroup = unitGroup
+        self.connect(recentButton, QtCore.SIGNAL('clicked()'), self.recentMenu)
+        extraLayout.addWidget(recentButton)
+        self.textButtons.append(recentButton)
+        self.recentButtons.append(recentButton)
+        self.setRecentAvail()
+
+    def recentMenu(self):
+        """Show a menu with recently used units"""
+        button = self.sender()
+        menu = QtGui.QMenu()
+        for unit in self.recentUnits:
+            action = menu.addAction(unit)
+            action.unitGroup = button.unitGroup
+        self.connect(menu, QtCore.SIGNAL('triggered(QAction*)'),
+                     self.insertRecent)
+        menu.exec_(button.mapToGlobal(QtCore.QPoint(0, 0)))
+
+    def setRecentAvail(self):
+        """Enable or disable recent unit button"""
+        for button in self.recentButtons:
+            button.setEnabled(len(self.recentUnits))
+
+    def insertRecent(self, action):
+        """Insert the recent unit from the given action"""
+        action.unitGroup.update(unicode(action.text()))
+        if action.unitGroup is self.fromGroup:
+            self.fromUnitEdit.unitUpdate()
+            self.fromUnitListView.updateSelection()
+        else:
+            self.toUnitEdit.unitUpdate()
+            self.toUnitListView.updateSelection()
 
     def updateColors(self):
         """Adjust the colors to the current option settings"""
@@ -219,21 +270,26 @@ class ConvertDlg(QtGui.QWidget):
         origBackground = self.getOptionColor('Background')
         origForeground = self.getOptionColor('Foreground')
         dlg = optiondlg.OptionDlg(self.option, self)
-        dlg.startGroupBox('Result Display')
-        optiondlg.OptionDlgInt(dlg, 'DecimalPlaces', 'Decimal places',
+        dlg.startGroupBox(_('Result Display'))
+        optiondlg.OptionDlgInt(dlg, 'DecimalPlaces', _('Decimal places'),
                                0, UnitGroup.maxDecPlcs)
-        optiondlg.OptionDlgBool(dlg, 'SciNotation', 'Use scientific notation')
+        optiondlg.OptionDlgBool(dlg, 'SciNotation',
+                                _('Use scientific notation'))
         optiondlg.OptionDlgBool(dlg, 'FixedDecimals',
-                                'Use fixed decimal places')
-        dlg.startGroupBox('Buttons')
-        optiondlg.OptionDlgBool(dlg, 'ShowOpButtons', 'Show operator buttons')
-        dlg.startGroupBox('Colors')
+                                _('Use fixed decimal places'))
+        dlg.startGroupBox(_('Recent Units'))
+        optiondlg.OptionDlgInt(dlg, 'RecentUnits', _('Number saved'), 0, 99)
+        dlg.startGroupBox(_('Buttons'))
+        optiondlg.OptionDlgBool(dlg, 'ShowOpButtons',
+                                _('Show operator buttons'))
+        dlg.startGroupBox(_('Colors'))
         optiondlg.OptionDlgBool(dlg, 'UseDefaultColors',
-                                'Use default system colors')
-        optiondlg.OptionDlgPush(dlg, 'Set background color', self.backColor)
-        optiondlg.OptionDlgPush(dlg, 'Set text color', self.textColor)
+                                _('Use default system colors'))
+        optiondlg.OptionDlgPush(dlg, _('Set background color'), self.backColor)
+        optiondlg.OptionDlgPush(dlg, _('Set text color'), self.textColor)
         if dlg.exec_() == QtGui.QDialog.Accepted:
             self.option.writeChanges()
+            self.recentUnits.updateQuantity()
             self.updateColors()
             self.showHideButtons()
             self.fromNumEdit.unitUpdate()
@@ -283,15 +339,16 @@ class ConvertDlg(QtGui.QWidget):
         if modPath.endswith('.zip'):  # for py2exe
             modPath = os.path.dirname(modPath)
         pathList = [helpFilePath, os.path.join(modPath, '../doc/'),
-                    modPath, 'doc/']
+                    modPath, os.path.join(modPath, 'doc/')]
+        fileList = ['README.html']
+        if lang and lang != 'C':
+            fileList[0:0] = ['README_%s.html' % lang,
+                             'README_%s.html' % lang[:2]]
         for path in filter(None, pathList):
-            try:
-                fullPath = os.path.join(path, 'README.html')
-                f = file(fullPath, 'r')
-                f.close()
-                return fullPath
-            except IOError:
-                pass
+            for fileName in fileList:
+                fullPath = os.path.join(path, fileName)
+                if os.access(fullPath, os.R_OK):
+                    return fullPath
         return ''
 
     def help(self):
@@ -300,16 +357,16 @@ class ConvertDlg(QtGui.QWidget):
             path = self.findHelpFile()
             if not path:
                 QtGui.QMessageBox.warning(self, 'ConvertAll',
-                                          'Read Me file not found')
+                                          _('Read Me file not found'))
                 return
-            self.helpView = helpview.HelpView(path, 'ConvertAll README File',
+            self.helpView = helpview.HelpView(path, _('ConvertAll README File'),
                                               self.icons)
         self.helpView.show()
 
     def about(self):
         """Show about info"""
         QtGui.QMessageBox.about(self, 'ConvertAll',
-                                'ConvertAll Version %s\nby %s' %
+                                _('ConvertAll Version %s\nby %s') %
                                 (__version__, __author__))
 
     def closeEvent(self, event):
@@ -323,5 +380,6 @@ class ConvertDlg(QtGui.QWidget):
             self.option.changeData('FinderYSize', self.findDlg.height(), True)
             self.option.changeData('FinderXPos', self.findDlg.x(), True)
             self.option.changeData('FinderYPos', self.findDlg.y(), True)
+        self.recentUnits.writeList()
         self.option.writeChanges()
         event.accept()
