@@ -22,13 +22,14 @@ import unitdata
 
 class NumEdit(QtGui.QLineEdit):
     """Number entry editor"""
-    def __init__(self, thisUnit, otherUnit, label, status, primary,
-                 parent=None):
+    def __init__(self, thisUnit, otherUnit, label, status, recentUnits,
+                 primary, parent=None):
         QtGui.QLineEdit.__init__(self, parent)
         self.thisUnit = thisUnit
         self.otherUnit = otherUnit
         self.label = label
         self.status = status
+        self.recentUnits = recentUnits
         self.primary = primary
         self.onLeft = primary
         self.setValidator(FloatExprValidator(self))
@@ -46,50 +47,53 @@ class NumEdit(QtGui.QLineEdit):
                     self.otherUnit.reduceGroup()
                 except unitdata.UnitDataError, text:
                     QtGui.QMessageBox.warning(self, 'ConvertAll',
-                                              'Error in unit data - %s' % text)
+                                              _('Error in unit data - %s')
+                                              % text)
                     return
                 if self.thisUnit.categoryMatch(self.otherUnit):
-                    self.status.setText('Converting...')
+                    self.status.setText(_('Converting...'))
                     if self.primary:
                         self.convert()
                     else:
                         self.emit(QtCore.SIGNAL('convertRqd'))
                     return
                 if self.onLeft:
-                    self.status.setText('Units are not compatible '\
-                               '(%s  vs.  %s)' % (self.thisUnit.compatStr(),
+                    self.status.setText(_(u'Units are not compatible '\
+                              '(%s  vs.  %s)') % (self.thisUnit.compatStr(),
                                                   self.otherUnit.compatStr()))
                 else:
-                    self.status.setText('Units are not compatible '\
-                               '(%s  vs.  %s)' % (self.otherUnit.compatStr(),
+                    self.status.setText(_('Units are not compatible '\
+                              '(%s  vs.  %s)') % (self.otherUnit.compatStr(),
                                                   self.thisUnit.compatStr()))
             else:
-                self.status.setText('Set units')
+                self.status.setText(_('Set units'))
         else:
-            self.status.setText('Set units')
-            self.label.setTitle('No Unit Set')
+            self.status.setText(_('Set units'))
+            self.label.setTitle(_('No Unit Set'))
         self.setEnabled(False)
-        self.emit(QtCore.SIGNAL('convertNum(QString &)'), QtCore.QString())
+        self.emit(QtCore.SIGNAL('convertNum'), '')
 
     def convert(self):
         """Do conversion with self primary"""
         self.primary = True
         self.setEnabled(True)
+        self.recentUnits.addEntry(self.thisUnit.unitString())
+        self.recentUnits.addEntry(self.otherUnit.unitString())
         try:
-            num = float(eval(str(self.text())))
+            num = float(eval(unicode(self.text())))
         except:
-            self.emit(QtCore.SIGNAL('convertNum(QString &)'), '')
+            self.emit(QtCore.SIGNAL('convertNum'), '')
             return
         try:
             numText = self.thisUnit.convertStr(num, self.otherUnit)
-            self.emit(QtCore.SIGNAL('convertNum(QString &)'), numText)
+            self.emit(QtCore.SIGNAL('convertNum'), numText)
         except unitdata.UnitDataError, text:
             QtGui.QMessageBox.warning(self, 'ConvertAll',
-                                      'Error in unit data - %s' % text)
+                                      _('Error in unit data - %s') % text)
 
     def setNum(self, numText):
         """Set text based on conversion from other number editor"""
-        if numText.isNull():
+        if not numText:
             self.setEnabled(False)
         else:
             self.primary = False
@@ -105,6 +109,6 @@ class FloatExprValidator(QtGui.QValidator):
 
     def validate(self, inputStr, pos):
         """Check for valid characters in entry"""
-        if FloatExprValidator.invalidRe.search(str(inputStr)):  # invalid char
+        if FloatExprValidator.invalidRe.search(unicode(inputStr)):
             return (QtGui.QValidator.Invalid, pos)
         return (QtGui.QValidator.Acceptable, pos)
