@@ -14,10 +14,12 @@
 
 import sys
 import os.path
+import codecs
 try:
-    from __main__ import dataFilePath
+    from __main__ import dataFilePath, lang
 except ImportError:
     dataFilePath = None
+    lang = ''
 import unitatom
 
 
@@ -33,31 +35,34 @@ class UnitData(dict):
         self.sortedKeys = []
         self.typeList = []
 
-    def findDataFile(self, pathList):
-        """Search paths for file, return line list or None"""
-        for path in pathList:
-            try:
-                f = file(os.path.join(path, 'units.dat'), 'r')
-                lineList = f.readlines()
-                f.close()
-                return lineList
-            except IOError:
-                pass
-        raise UnitDataError, 'Can not read "units.dat" file'
-
-    def readData(self):
-        """Read all unit data from file, return number loaded"""
+    def findDataFile(self):
+        """Search for data file, return line list or None"""
         modPath = os.path.abspath(sys.path[0])
         if modPath.endswith('.zip'):  # for py2exe
             modPath = os.path.dirname(modPath)
-        pathList = [dataFilePath, os.path.join(modPath, '../data/'),
-                    modPath]
-        lines = self.findDataFile(filter(None, pathList))
+        pathList = [dataFilePath, os.path.join(modPath, '../data/'), modPath]
+        fileList = ['units.dat']
+        if lang and lang != 'C':
+            fileList[0:0] = ['units_%s.dat' % lang, 'units_%s.dat' % lang[:2]]
+        for path in filter(None, pathList):
+            for fileName in fileList:
+                try:
+                    f = codecs.open(os.path.join(path, fileName), 'r', 'utf-8')
+                    lineList = f.readlines()
+                    f.close()
+                    return lineList
+                except IOError:
+                    pass
+        raise UnitDataError, _('Can not read "units.dat" file')
+
+    def readData(self):
+        """Read all unit data from file, return number loaded"""
+        lines = self.findDataFile()
         for i in range(len(lines)):     # join continuation lines
             delta = 1
             while lines[i].rstrip().endswith('\\'):
-                lines[i] = ''.join([lines[i].rstrip()[:-1], lines[i+delta]])
-                lines[i+delta] = ''
+                lines[i] = u''.join([lines[i].rstrip()[:-1], lines[i+delta]])
+                lines[i+delta] = u''
                 delta += 1
         units = [unitatom.UnitAtom(line) for line in lines if
                  line.split('#', 1)[0].strip()]   # remove comment lines
@@ -73,7 +78,7 @@ class UnitData(dict):
         self.sortedKeys = self.keys()
         self.sortedKeys.sort()
         if len(self.sortedKeys) < len(units):
-            raise UnitDataError, 'Duplicate unit names found'
+            raise UnitDataError, _('Duplicate unit names found')
         return len(units)
 
     def findPartialMatch(self, text):
