@@ -4,7 +4,7 @@
 # unitgroup.py, provides a group of units and does conversions
 #
 # ConvertAll, a units conversion program
-# Copyright (C) 2006, Douglas W. Bell
+# Copyright (C) 2010, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
@@ -21,7 +21,8 @@ import unitdata
 class UnitGroup(object):
     """Stores, updates and converts a group of units"""
     maxDecPlcs = 12
-    operRegEx = re.compile(r'([\*/])')
+    operRegEx = re.compile(r'[\*/]')
+    operGroupRegEx = re.compile(r'(\(.*\)|\(.*$|[\*/])')
     def __init__(self, unitData, option):
         self.unitData = unitData
         self.option = option
@@ -30,6 +31,44 @@ class UnitGroup(object):
         self.factor = 1.0
         self.reducedList = []
         self.linear = True
+        self.parenthClosed = True
+
+    def flatUnitList(self):
+        """Return the units with sub-groups flattened"""
+        result = []
+        for unit in self.unitList:
+            if hasattr(unit, 'flatUnitList'):
+                result.extend(unit.flatUnitList())
+            else:
+                result.append(unit)
+        return result
+
+    def unitGroups(self):
+        """Return a list of this unit group and all sub-groups"""
+        result = [self]
+        for group in self.unitList:
+            if hasattr(group, 'unitGroups'):
+                result.extend(group.unitGroups())
+        return result
+
+    def unitGroupExpSign(self):
+        """Return True if the first unit's exponent is positive"""
+        unitList = self.unitList
+        while unitList and not hasattr(unitList[0], 'exp'):
+            unitList = unitList[0].unitList
+        if unitList and unitList[0].exp < 0:
+            return False
+        return True
+
+    def currentGroupPos(self):
+        """Return a tuple of the group and position of the current unit"""
+        currentUnit = self.currentUnit()
+        if currentUnit:
+            for group in self.unitGroups():
+                for i in range(len(group.unitList)):
+                    if group.unitList[i] is currentUnit:
+                        return (group, i)
+        return (self, 0)
 
     def update(self, text, cursorPos=None):
         """Decode user entered text into units"""
@@ -37,48 +76,50 @@ class UnitGroup(object):
         if cursorPos != None:
             self.updateCurrentUnit(text, cursorPos)
         else:
-            self.currentNum = len(self.unitList) - 1
+            self.currentNum = len(self.flatUnitList()) - 1
 
     def updateCurrentUnit(self, text, cursorPos):
         """Set current unit number"""
         self.currentNum = len(UnitGroup.operRegEx.findall(text[:cursorPos]))
 
     def currentUnit(self):
-        """Return current unit if its a full match, o/w None"""
-        if self.unitList and self.unitList[self.currentNum].equiv:
-            return self.unitList[self.currentNum]
-        return None
+        """Return current unit if set, o/w None"""
+        try:
+            return self.flatUnitList()[self.currentNum]
+        except IndexError:
+            return None
 
     def currentPartialUnit(self):
         """Return unit with at least a partial match, o/w None"""
-        if not self.unitList:
+        currentUnit = self.currentUnit()
+        if not currentUnit:
             return None
-        return self.unitData.findPartialMatch(self.unitList[self.currentNum]\
-                                              .name)
+        return self.unitData.findPartialMatch(currentUnit.name)
 
     def currentSortPos(self):
         """Return unit near current unit for sorting"""
-        if not self.unitList:
+        try:
+            return self.unitData.findSortPos(self.currentUnit().name)
+        except AttributeError:
             return self.unitData[self.unitData.sortedKeys[0]]
-        return self.unitData.findSortPos(self.unitList[self.currentNum].
-                                         name)
 
-    def replaceCurrent(self, unit):
+    def replaceCurrent(self, newUnit):
         """Replace the current unit with unit"""
         if self.unitList:
-            exp = self.unitList[self.currentNum].exp
-            self.unitList[self.currentNum] = unit.copy()
-            self.unitList[self.currentNum].exp = exp
-        else:
-            self.unitList.append(unit.copy())
+            oldUnit = self.currentUnit()
+            group, pos = self.currentGroupPos()
+            group.unitList[pos] = newUnit.copy()
+            group.unitList[pos].exp = oldUnit.exp
+            return
+        self.unitList.append(newUnit.copy())
 
     def completePartial(self):
         """Replace a partial unit with a full one"""
-        if self.unitList and not self.unitList[self.currentNum].equiv:
-            text = self.unitList[self.currentNum].name
-            unit = self.unitData.findPartialMatch(text)
-            if unit:
-                self.replaceCurrent(unit)
+        partUnit = self.currentUnit()
+        if partUnit and not partUnit.equiv:
+            newUnit = self.unitData.findPartialMatch(partUnit.name)
+            if newUnit:
+                self.replaceCurrent(newUnit)
 
     def moveToNext(self, upward):
         """Replace unit with adjacent one based on match or sort position"""
@@ -92,20 +133,21 @@ class UnitGroup(object):
         """Add new operator & blank unit after current, * if mult is true"""
         if self.unitList:
             self.completePartial()
-            prevExp = self.unitList[self.currentNum].exp
+            group, pos = self.currentGroupPos()
             self.currentNum += 1
-            self.unitList.insert(self.currentNum, UnitAtom(''))
-            if (not mult and prevExp > 0) or (mult and prevExp < 0):
-                self.unitList[self.currentNum].exp = -1
+            group.unitList.insert(pos + 1, UnitAtom(u''))
+            if not mult:
+                self.currentUnit().exp = -1
 
     def changeExp(self, newExp):
         """Change the current unit's exponent"""
-        if self.unitList:
-            self.completePartial()
-            if self.unitList[self.currentNum].exp > 0:
-                self.unitList[self.currentNum].exp = newExp
+        self.completePartial()
+        currentUnit = self.currentUnit()
+        if currentUnit:
+            if currentUnit.exp > 0:
+                currentUnit.exp = newExp
             else:
-                self.unitList[self.currentNum].exp = -newExp
+                currentUnit.exp = -newExp
 
     def clearUnit(self):
         """Remove units"""
@@ -118,15 +160,41 @@ class UnitGroup(object):
     def parseGroup(self, text):
         """Return list of units from text string"""
         unitList = []
-        parts = [part.strip() for part in UnitGroup.operRegEx.split(text)]
+        parts = [part.strip() for part in UnitGroup.operGroupRegEx.split(text)
+                 if part.strip()]
         numerator = True
         while parts:
-            unit = self.parseUnit(parts.pop(0))
-            if not numerator:
-                unit.exp = -unit.exp
-            if parts and parts.pop(0) == '/':
-                numerator = not numerator
-            unitList.append(unit)
+            part = parts.pop(0)
+            if part == '*' or part  == '/':
+                parts.insert(0, part)
+                part = u''      # add blank invalid unit if order wrong
+            if part.startswith('('):
+                part = part[1:]
+                group = UnitGroup(self.unitData, self.option)
+                if part.endswith(')'):
+                    part = part[:-1]
+                else:
+                    group.parenthClosed = False
+                group.update(part)
+                if not group.unitList:
+                    group.unitList.append(group.parseUnit(''))
+                if not numerator:
+                    for unit in group.flatUnitList():
+                        unit.exp = -unit.exp
+                unitList.append(group)
+            else:
+                unit = self.parseUnit(part)
+                if not numerator:
+                    unit.exp = -unit.exp
+                unitList.append(unit)
+            if parts:
+                oper = parts.pop(0)
+                if oper == '*' or oper == '/':
+                    numerator = oper == '*' and True or False
+                    if not parts:
+                        parts.insert(0, u'')  # add blank invalid unit at end
+                else:
+                    parts.insert(0, oper)  # put unit back if order wrong
         return unitList
 
     def parseUnit(self, text):
@@ -147,35 +215,54 @@ class UnitGroup(object):
            self.unitData.findPartialMatch(unitText):   # check for plural
             unit = self.unitData.get(unitText[:-1], None)
         if not unit:
-            unit = UnitAtom(parts[0].strip())   # tmp invalid unit
+            unit = UnitAtom('')   # tmp invalid unit
+            unit.name = parts[0].strip()
         unit = unit.copy()
         unit.exp = exp
         return unit
 
-    def unitString(self, unitList=None):
+    def unitString(self, unitList=None, swapExpSign=False):
         """Return the full string for this group or a given group"""
         if unitList == None:
             unitList = self.unitList
         fullText = ''
         if unitList:
-            fullText = unitList[0].unitText(0)
-            numerator = True
-            for unit in unitList[1:]:
-                if (numerator and unit.exp > 0) \
-                   or (not numerator and unit.exp < 0):
-                    fullText = '%s * %s' % (fullText, unit.unitText(1))
+            firstUnit = True
+            for unit in unitList:
+                if not firstUnit:
+                    if hasattr(unit, 'exp'):
+                        expSign = unit.exp > 0
+                    else:
+                        expSign = unit.unitGroupExpSign()
+                    if swapExpSign:
+                        expSign = not expSign
+                    fullText = u'%s %s ' % (fullText, expSign and '*' or '/')
+                if hasattr(unit, 'unitText'):
+                    fullText = u'%s%s' % (fullText,
+                                          unit.unitText(swapExpSign or
+                                                        not firstUnit))
                 else:
-                    fullText = '%s / %s' % (fullText, unit.unitText(1))
-                    numerator = not numerator
+                    if firstUnit and not swapExpSign:
+                        swap = False
+                    else:
+                        swap = not unit.unitGroupExpSign()
+                    fullText = u'%s(%s%s' % (fullText,
+                                             unit.unitString(None, swap),
+                                             unit.parenthClosed and ')' or '')
+                firstUnit = False
         return fullText
 
     def groupValid(self):
-        """Return True if all unitself.reducedLists are valid"""
-        if not self.unitList:
+        """Return True if all units are valid"""
+        if not self.unitList or not self.parenthClosed:
             return False
         for unit in self.unitList:
-            if not unit.unitValid():
-                return False
+            if hasattr(unit, 'unitValid'):
+                if not unit.unitValid():
+                    return False
+            else:
+                if not unit.groupValid():
+                    return False
         return True
 
     def reduceGroup(self):
@@ -186,21 +273,23 @@ class UnitGroup(object):
         if not self.groupValid():
             return
         count = 0
-        tmpList = self.unitList[:]
+        tmpList = self.flatUnitList()
         while tmpList:
             count += 1
             if count > 5000:
-                raise unitdata.UnitDataError, 'Circular unit definition'
+                raise unitdata.UnitDataError, _('Circular unit definition')
             unit = tmpList.pop(0)
-            if unit.equiv == '!':
+            if unit.equiv.startswith('!'):
                 self.reducedList.append(unit.copy())
             elif not unit.equiv:
                 raise unitdata.UnitDataError, \
-                      'Invalid conversion for "%s"' % unit.name
+                      _('Invalid conversion for "%s"') % unit.name
             else:
                 if unit.fromEqn:
                     self.linear = False
-                newList = self.parseGroup(unit.equiv)
+                equivUnit = UnitGroup(self.unitData, self.option)
+                equivUnit.update(unit.equiv)
+                newList = equivUnit.flatUnitList()
                 for newUnit in newList:
                     newUnit.exp *= unit.exp
                 tmpList.extend(newList)
@@ -214,7 +303,8 @@ class UnitGroup(object):
             else:
                 self.reducedList.append(unit)
         self.reducedList = [unit for unit in self.reducedList if
-                            unit.name != 'unit' and unit.exp != 0]
+                            unit.equiv != '!!' and unit.name != u'unit' and
+                            unit.exp != 0]
 
     def categoryMatch(self, otherGroup):
         """Return True if unit types are equivalent"""
@@ -227,7 +317,8 @@ class UnitGroup(object):
     def checkLinear(self):
         """Return True if linear or acceptable non-linear"""
         if not self.linear:
-            if len(self.unitList) > 1 or self.unitList[0].exp != 1:
+            flatList = self.flatUnitList()
+            if len(flatList) > 1 or flatList[0].exp != 1:
                 return False
         return True
 
@@ -235,7 +326,7 @@ class UnitGroup(object):
         """Return string with reduced unit or linear compatability problem"""
         if self.checkLinear():
             return self.unitString(self.reducedList)
-        return 'Cannot combine non-linear units'
+        return _('Cannot combine non-linear units')
 
     def convert(self, num, toGroup):
         """Return num of this group converted to toGroup"""
@@ -251,11 +342,12 @@ class UnitGroup(object):
         """Return result of non-linear calculation"""
         x = num
         try:
-            if self.unitList[0].toEqn:      # regular equations
+            unit = self.flatUnitList()[0]
+            if unit.toEqn:      # regular equations
                 if isFrom:
-                    return float(eval(self.unitList[0].fromEqn))
-                return float(eval(self.unitList[0].toEqn))
-            data = list(eval(self.unitList[0].fromEqn))  # extrapolation list
+                    return float(eval(unit.fromEqn))
+                return float(eval(unit.toEqn))
+            data = list(eval(unit.fromEqn))  # extrapolation list
             if isFrom:
                 data = [(float(group[0]), float(group[1])) for group in data]
             else:
@@ -274,7 +366,7 @@ class UnitGroup(object):
             return 1e9999
         except:
             raise unitdata.UnitDataError, \
-                  'Bad equation for %s' % self.unitList[0].name
+                  _('Bad equation for %s') % unit.name
 
     def convertStr(self, num, toGroup):
         """Return formatted string of converted number"""
@@ -284,10 +376,10 @@ class UnitGroup(object):
         """Return num string formatted per options"""
         decPlcs = self.option.intData('DecimalPlaces', 0, UnitGroup.maxDecPlcs)
         if self.option.boolData('SciNotation'):
-            return ('%%0.%dE' % decPlcs) % num
+            return (u'%%0.%dE' % decPlcs) % num
         if self.option.boolData('FixedDecimals'):
-            return ('%%0.%df' % decPlcs) % num
-        return ('%%0.%dG' % decPlcs) % num
+            return (u'%%0.%df' % decPlcs) % num
+        return (u'%%0.%dG' % decPlcs) % num
 
 
 if __name__ == '__main__':
@@ -305,15 +397,15 @@ if __name__ == '__main__':
     toText = raw_input('Enter to unit -> ')
     toUnit = UnitGroup(data, options)
     toUnit.update(toText)
-    print '%s   TO   %s' % (fromUnit.unitString(), toUnit.unitString())
+    print u'%s   TO   %s' % (fromUnit.unitString(), toUnit.unitString())
     fromUnit.reduceGroup()
     toUnit.reduceGroup()
-    print '%s   TO   %s' % (fromUnit.unitString(fromUnit.reducedList),
-                            toUnit.unitString(toUnit.reducedList))
+    print u'%s   TO   %s' % (fromUnit.unitString(fromUnit.reducedList),
+                             toUnit.unitString(toUnit.reducedList))
     if not fromUnit.categoryMatch(toUnit):
         print 'NO MATCH'
     else:
         print 'MATCH'
         numText = raw_input('Enter value -> ')
         num = float(numText)
-        print '%f   IS  %f' % (num, fromUnit.convert(num, toUnit))
+        print u'%f   IS  %f' % (num, fromUnit.convert(num, toUnit))
